#ifndef __DBQUERY_H__
#define __DBQUERY_H__

#include <time.h>
#include <stdlib.h>
#include "String.h"

////////////////////////////////////////////////////////////////////////////////////////////
// class DbQuery - generic interface to a reader from database
////////////////////////////////////////////////////////////////////////////////////////////
class DbQuery {
protected:
        virtual const char*     get_value( const char* fname, const char* defaultValue ) {
                return NULL;
        }
public:
        // methods to be overwritten by descendants
        virtual bool            eof() = 0;
        virtual CString         fetch() = 0;
        // methods optionally overritten by descendants
        virtual bool next() {
                return (fetch()=="") && !eof();
        }
        virtual const char* get_string( const char* fname, const char* defaultValue ) {
                return get_value(fname,defaultValue);
        }
        virtual int get_int( const char* fname, int defaultValue ) {
                const char* value = get_value(fname,NULL);
                if( !value ) return defaultValue;
                char* end;
                int result = strtol(value,&end,0);
                return *end ? defaultValue : result;
        }
        virtual bool get_bool( const char* fname, bool defaultValue ) {
                const char* value = get_value(fname,NULL);
                if( !value ) return defaultValue;
                return (*value=='t') ? true : (*value=='f') ? false : defaultValue;
        }
        virtual time_t get_time( const char* fname, time_t defaultValue ) {
                const char* value = get_value(fname,NULL);
                if( !value ) return defaultValue;
                struct tm tm;
                char* end = strptime(value,"%Y-%m-%d %H:%M:%S",&tm);
                if( *end && (*end!='.') ) return defaultValue;
                return mktime(&tm);
        }
};

#endif // __DBQUERY_H__
