#ifndef __ODBCQUERY_H__
#define __ODBCQUERY_H__

#include <sql.h>
#include <sqlext.h>
#include "DbQuery.h"
#include "Vector.h"

////////////////////////////////////////////////////////////////////////////////////////////
// functions
////////////////////////////////////////////////////////////////////////////////////////////
CString getODBCErrorText( SQLSMALLINT handleType, SQLHANDLE handle );

////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCEnvironment
////////////////////////////////////////////////////////////////////////////////////////////
class ODBCEnvironment {
private:
        SQLHANDLE m_henv;

public:
        // methods
        operator SQLHANDLE()    const { return m_henv; }
        CString getErrorText()  const { return getODBCErrorText(SQL_HANDLE_ENV,m_henv); }
        ODBCEnvironment& operator=( ODBCEnvironment& e ) {
                close();
                m_henv = e.m_henv;
                e.m_henv = 0;
                return *this;
        }
        CString open( bool connectionPooling=false );
        void close();
        bool is_open() const { return m_henv!=0; }
        // constructor/destructor
        ODBCEnvironment() {
                m_henv = 0;
        }
        ~ODBCEnvironment() {
                close();
        }
};

////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCConnection
////////////////////////////////////////////////////////////////////////////////////////////
class ODBCConnection {
private:
        ODBCEnvironment         m_env;
        SQLHANDLE               m_hdbc;

public:
        // methods
        const ODBCEnvironment& getEnvironment() const { return m_env; }
        operator SQLHANDLE()                    const { return m_hdbc; }
        /* bad design here */
        ODBCConnection& operator=( ODBCConnection& c ) {
                close();
                m_env = c.m_env;
                m_hdbc = c.m_hdbc;
                c.m_hdbc = 0;
                return *this;
        }
        CString getErrorText()  const { return getODBCErrorText(SQL_HANDLE_DBC,m_hdbc); }
        CString open( const char* connectionString, bool connectionPooling=false, bool writable=true );
        void    close();
        bool    is_open() const { return (m_hdbc!=0) && m_env.is_open(); }
        // constructor/destructor
        ODBCConnection() {
                m_hdbc = 0;
        }
        ~ODBCConnection() {
                close();
        }
};

////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCQuery
////////////////////////////////////////////////////////////////////////////////////////////
class ODBCQuery : public DbQuery {
private:
        // types
        struct ColumnInfo {
                CString         m_name;
                SQLSMALLINT     m_type;
                int             m_ndx;
                ColumnInfo( const char* name, SQLSMALLINT type, int ndx ) : m_name(name),m_type(type),m_ndx(ndx) {
                }
        };
        // properties
        HSTMT                   m_hstmt;
        bool                    m_eof;
        char*                   m_string_buff;
        Vector<ColumnInfo,100>  m_names_to_ordinalsMap;
        // methods
        static int _compare_fields_by_name( const void* arg, const ColumnInfo* columnInfo ) {
#if DEBUG
                fprintf("ODBCQuery::_compare_fields_by_name:arg=%s,columnInfo->m_name=%s\n",arg,(const char*)columnInfo->m_name);
#endif
                return strcmp((const char*)arg,columnInfo->m_name);
        };
        CString build_name_to_ordinals_map();

public:
        // methods orverloaded from DbQuery
        bool            eof() { return m_eof; }
        CString         fetch();
        const char*     get_string( const char* fname, const char* defaultValue );
        int             get_int( const char* fname, int defaultValue );
        bool            get_bool( const char* fname, bool defaultValue );
        time_t          get_time( const char* fname, time_t defaultValue );
        // own methods
        CString         getErrorText()  const { return getODBCErrorText(SQL_HANDLE_STMT,m_hstmt); }
        CString         open( const ODBCConnection& dbconn, const char* query );
        void            close();
        bool            is_open() const { return m_hstmt!=0; }
        bool            got_column( const char* name );
        // constructor/destructor
        ODBCQuery() {
                m_hstmt = 0;
                m_eof = false;
                m_string_buff = 0;
        }
        virtual ~ODBCQuery() {
                close();
        }
};

#endif  // __ODBCQUERY_H__
