#include <stdio.h>
#include "ODBCQuery.h"

////////////////////////////////////////////////////////////////////////////////////////////
// functions
////////////////////////////////////////////////////////////////////////////////////////////
CString getODBCErrorText( SQLSMALLINT handleType, SQLHANDLE handle ) {
        CString         result;
        SQLCHAR         szErrState[SQL_SQLSTATE_SIZE+1];       // SQL Error State string
        SQLINTEGER      nativeErrCode;                         // Native Error code
        UCHAR           szErrText[SQL_MAX_MESSAGE_LENGTH+1];   // SQL Error Text string
        SQLSMALLINT     wErrMsgLen;
        // call SQLGetDiagRec function with proper ODBC handles, repeatedly until
        // function returns SQL_NO_DATA_FOUND. Concatenate all error strings
        // in the display buffer and display all results.
        for( SQLSMALLINT recNumber=1; ; recNumber++ ) {
                SQLRETURN n = SQLGetDiagRec(handleType,handle,recNumber,szErrState,&nativeErrCode,szErrText,sizeof(szErrText),&wErrMsgLen);
                if( (n!=SQL_SUCCESS) && (n!=SQL_SUCCESS_WITH_INFO) )
                        break;
                result = formatString("%sSQL Error State:%s, Native Error Code: %lX, ODBC Error: %s;",(const char*)result,(char*)szErrState,nativeErrCode,(char*)szErrText);
        }
        return result;
}
////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCEnvironment
////////////////////////////////////////////////////////////////////////////////////////////
CString ODBCEnvironment::open( bool connectionPooling ) {
        CString result;
        if( SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&m_henv)==SQL_SUCCESS ) {
                if( SQLSetEnvAttr(m_henv,SQL_ATTR_ODBC_VERSION,(void*)SQL_OV_ODBC3,0)==SQL_SUCCESS ) {
                        if( !connectionPooling || SQLSetEnvAttr(m_henv,SQL_ATTR_CONNECTION_POOLING,(void*)SQL_CP_ONE_PER_DRIVER,0)==SQL_SUCCESS ) {
                                return "";
                        }
                        else {
                                result = formatString("SQLSetEnvAttr(SQL_ATTR_CONNECTION_POOLING) failed (%s)",(const char*)getErrorText());
                        }
                }
                else {
                        result = formatString("SQLSetEnvAttr(SQL_ATTR_ODBC_VERSION) failed (%s)",(const char*)getErrorText());
                }
                close();
        }
        else {
                result = formatString("SQLAllocHandle failed (%s)",(const char*)getODBCErrorText(SQL_HANDLE_ENV,SQL_NULL_HANDLE));
        }
        return result;
}
void ODBCEnvironment::close() {
        if( m_henv ) {
                SQLFreeHandle(SQL_HANDLE_ENV,m_henv);
                m_henv = 0;
        }
}
////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCConnection
////////////////////////////////////////////////////////////////////////////////////////////
CString ODBCConnection::open( const char* connectionString, bool connectionPooling, bool writable ) {
        CString result;
        if( (result=m_env.open(connectionPooling))=="" ) {
                SQLRETURN retcode = SQLAllocHandle(SQL_HANDLE_DBC,m_env,&m_hdbc);
                if( retcode==SQL_SUCCESS ) {
                        SQLCHAR outConnectionString[1024];
                        SQLSMALLINT outConnectionStringLength;
                        // if failed to connect, free the allocated hdbc before return
                        RETCODE retcode = SQLDriverConnect(m_hdbc,0,
                                                           (SQLCHAR*)connectionString,strlen(connectionString),
                                                           outConnectionString,sizeof(outConnectionString),&outConnectionStringLength,
                                                           SQL_DRIVER_NOPROMPT);
                        if( (retcode==SQL_SUCCESS) || (retcode==SQL_SUCCESS_WITH_INFO) ) {
                                if( !writable || (SQLSetConnectAttr(m_hdbc,SQL_ATTR_ACCESS_MODE,SQL_MODE_READ_WRITE,0)==SQL_SUCCESS) ) {
                                        return "";
                                }
                                else {
                                        result = formatString("SQLSetConnectAttr(SQL_ATTR_ACCESS_MODE) failed (%s)",(const char*)getErrorText());
                                }
                        }
                        else {
                                result = formatString("SQLDriverConnect failed (%s)",(const char*)getErrorText());
                        }
                        SQLFreeHandle(SQL_HANDLE_DBC,m_hdbc);
                        m_hdbc = NULL;
                }
                else {
                        result = formatString("SQLAllocHandle failed (%s),retcode=%d,m_hdbc=%d",(const char*)m_env.getErrorText(),retcode,m_hdbc);
                }
                m_env.close();
        }
        else {
                result = formatString("Unable to init environment (%s)",(const char*)result);
        }
        return result;
}
void ODBCConnection::close() {
        m_env.close();
        if( m_hdbc ) {
                SQLDisconnect(m_hdbc);
                SQLFreeHandle(SQL_HANDLE_DBC,m_hdbc);
                m_hdbc = 0;
        }
}
////////////////////////////////////////////////////////////////////////////////////////////
// class ODBCQuery
////////////////////////////////////////////////////////////////////////////////////////////
CString ODBCQuery::build_name_to_ordinals_map() {
        CString result;
        SQLSMALLINT columns;
        SQLRETURN retcode = SQLNumResultCols(m_hstmt,&columns);
        if( (retcode==SQL_SUCCESS) || (retcode==SQL_SUCCESS_WITH_INFO) ) {
                for( SQLSMALLINT n=1; n<=columns; n++ ) {
                        char            name[256];
                        SQLSMALLINT     namesize;
                        SQLSMALLINT     type;
                        SQLUINTEGER     size;
                        SQLSMALLINT     decimalDigits;
                        SQLSMALLINT     nullable;
                        retcode = SQLDescribeCol(m_hstmt,n,(SQLCHAR*)name,sizeof(name),&namesize,&type,&size,&decimalDigits,&nullable);
                        if( (retcode==SQL_SUCCESS) || (retcode==SQL_SUCCESS_WITH_INFO) ) {
#if _DEBUG>1
                                fprintf(stderr,"ODBCQuery::build_name_to_ordinals_map:1.6:name=%s,type=%d,n=%d\n",name,type,n);
#endif
                                if( m_names_to_ordinalsMap.add(new ColumnInfo(name,type,n),m_names_to_ordinalsMap.bsearch_position(name,_compare_fields_by_name))<0 ) {
                                        result = formatString("Vector::add failed");
                                        break;
                                }
                        }
                        else {
                                result = formatString("SQLDescribeCol failed, n=%d (%s)",n,(const char*)getErrorText());
                                break;
                        }
                }
        }
        else {
                result = formatString("SQLNumResultCols failed (%s)",(const char*)getErrorText());
        }
        return result;
}
CString ODBCQuery::fetch() {
        CString result;
        if( m_names_to_ordinalsMap.size()==0 ) {
                if( (result=build_name_to_ordinals_map())!="" ) {
                        return formatString("Cannot build names to ordinals map (%s)",(const char*)result);
                }
        }
        SQLRETURN retcode = SQLFetch(m_hstmt);
        if( (retcode==SQL_SUCCESS) || (retcode==SQL_SUCCESS_WITH_INFO) ) {
                return "";
        }
        if( retcode==SQL_NO_DATA ) {
                m_eof = true;
                return "";
        }
        return formatString("SQLFetch failed (%s)",(const char*)getErrorText());
}
const char* ODBCQuery::get_string( const char* fname, const char* defaultValue ) {
        const char* result = defaultValue;
        ColumnInfo* columnInfo = m_names_to_ordinalsMap.bsearch(fname,_compare_fields_by_name);
        if( columnInfo ) {
                SQLINTEGER bufflength = 1024;
                if( m_string_buff || (m_string_buff=(char*)malloc(bufflength))!=NULL ) {
                        SQLINTEGER size;
                        if( SQLGetData(m_hstmt,columnInfo->m_ndx,SQL_C_CHAR,m_string_buff,bufflength,&size)==SQL_SUCCESS ) {
                                if( size!=SQL_NULL_DATA ) {
                                        result = m_string_buff;
                                }
                        }
                }
        }
        return result;
}
int ODBCQuery::get_int( const char* fname, int defaultValue ) {
        int result = defaultValue;
        ColumnInfo* columnInfo = m_names_to_ordinalsMap.bsearch(fname,_compare_fields_by_name);
        if( columnInfo ) {
                SQLINTEGER slong;
                SQLINTEGER size;
                if( SQLGetData(m_hstmt,columnInfo->m_ndx,SQL_C_SLONG,&slong,sizeof(slong),&size)==SQL_SUCCESS ) {
                        if( size!=SQL_NULL_DATA ) {
                                result = slong;
                        }
                }
        }
        return result;
}
bool ODBCQuery::get_bool( const char* fname, bool defaultValue ) {
        bool result = defaultValue;
        ColumnInfo* columnInfo = m_names_to_ordinalsMap.bsearch(fname,_compare_fields_by_name);
        if( columnInfo ) {
                SQLCHAR bit;
                SQLINTEGER size;
                if( SQLGetData(m_hstmt,columnInfo->m_ndx,SQL_C_BIT,&bit,sizeof(bit),&size)==SQL_SUCCESS ) {
                        if( size!=SQL_NULL_DATA ) {
                                result = bit ? true : false;
                        }
                }
        }
        return result;
}
time_t ODBCQuery::get_time( const char* fname, time_t defaultValue ) {
        time_t result = defaultValue;
        ColumnInfo* columnInfo = m_names_to_ordinalsMap.bsearch(fname,_compare_fields_by_name);
        if( columnInfo ) {
                TIMESTAMP_STRUCT timestamp;
                SQLINTEGER size;
                if( SQLGetData(m_hstmt,columnInfo->m_ndx,SQL_C_TYPE_TIMESTAMP,&timestamp,sizeof(timestamp),&size)==SQL_SUCCESS ) {
                        if( size!=SQL_NULL_DATA ) {
                                tm tmm = {0};
                                tmm.tm_sec  = timestamp.second;
                                tmm.tm_min  = timestamp.minute;
                                tmm.tm_hour = timestamp.hour;
                                tmm.tm_mday = timestamp.day;
                                tmm.tm_mon  = timestamp.month;
                                tmm.tm_year = timestamp.year;
                                return mktime(&tmm);
                        }
                }
        }
        return result;
}
CString ODBCQuery::open( const ODBCConnection& dbconn, const char* query ) {
        CString result;
        SQLRETURN retcode = SQLAllocHandle(SQL_HANDLE_STMT,dbconn,&m_hstmt);
        if( retcode==SQL_SUCCESS ) {
                retcode = SQLExecDirect(m_hstmt,(SQLCHAR*)query,SQL_NTS);
                if( (retcode==SQL_SUCCESS) || (retcode==SQL_SUCCESS_WITH_INFO) || (retcode==SQL_NO_DATA) ) {
                        return "";
                }
                else {
                        result = formatString("SQLExecDirect returned %d (%s)",retcode,(const char*)getErrorText());
                }
                close();
        }
        else {
                result = formatString("SQLAllocStmt returned %d (%s)",retcode,(const char*)dbconn.getErrorText());
        }
        return result;
}
bool ODBCQuery::got_column( const char* fname ) {
        if( m_names_to_ordinalsMap.size()==0 ) {
                if( build_name_to_ordinals_map()!="" ) {
                        return false;
                }
        }
        return m_names_to_ordinalsMap.bsearch(fname,_compare_fields_by_name)!=NULL;
}
void ODBCQuery::close() {
        if( m_hstmt ) {
                SQLFreeStmt(m_hstmt,SQL_CLOSE);
                SQLFreeStmt(m_hstmt,SQL_DROP);
                m_hstmt = 0;
        }
        if( m_string_buff ) {
                free(m_string_buff);
                m_string_buff = 0;
        }
}
