#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include "String.h"

////////////////////////////////////////////////////////////////
// static data
////////////////////////////////////////////////////////////////
// #define HAVE_UGLY_NULL_FIX

////////////////////////////////////////////////////////////////
// static data
////////////////////////////////////////////////////////////////
#define afxEmptyString ((CString&)*(CString*)&_afxPchNil)
#define BYTE           unsigned char
#define ASSERT         assert
#define TCHAR          char

static int          _afxInitData[] = { -1, 0, 0, 0 };
static CStringData* _afxDataNil    = (CStringData*)&_afxInitData;
static const char*  _afxPchNil     = (const char*)(((BYTE*)&_afxInitData)+sizeof(CStringData));

long InterlockedIncrement( long* pLong ) {
  pLong[0]++;
  return pLong[0];
}

long InterlockedDecrement( long* pLong ) {
  pLong[0]--;
  return pLong[0];
}

////////////////////////////////////////////////////////////////
// class CString
////////////////////////////////////////////////////////////////
CString::CString() {
  Init();
}

CString::CString(const CString& stringSrc) {
  ASSERT(stringSrc.GetData()->nRefs != 0);
  if (stringSrc.GetData()->nRefs >= 0) {
    ASSERT(stringSrc.GetData() != _afxDataNil);
    m_pchData = stringSrc.m_pchData;
    InterlockedIncrement(&GetData()->nRefs);
  }
  else {
    Init();
    *this = stringSrc.m_pchData;
  }
}

CString::CString(const char* lpsz) {
  Init();
  int nLen = SafeStrlen(lpsz);
  if (nLen != 0) {
    AllocBuffer(nLen);
    memcpy(m_pchData, lpsz, nLen*sizeof(char));
  }
}

CString::CString( const char* lpsz, unsigned nLen )
{
  Init();
  if( nLen != 0) {
    AllocBuffer(nLen);
    memcpy(m_pchData,lpsz,nLen*sizeof(char));
  }
}

int CString::GetLength() const {
  return GetData()->nDataLength;
}

int CString::IsEmpty() const {
  return GetData()->nDataLength==0;
}

int CString::IsNotEmpty() const {
  return GetData()->nDataLength!=0;
}

void CString::Empty() {
  if (GetData()->nDataLength == 0)
    return;
  if (GetData()->nRefs >= 0)
    Release();
  else
    *this = "";
  ASSERT(GetData()->nDataLength == 0);
  ASSERT(GetData()->nRefs < 0 || GetData()->nAllocLength == 0);
}

CString CString::Trim() {
  const char* start;
  const char* end;
  for( start=m_pchData; isspace(*start); start++ );
  for( end  =m_pchData+this->GetLength()-1; isspace(*end); end-- );
  return CString(start,end-start+1);
}

CString CString::Replace( const char* pattern, const char* replacement, bool casesens, int howmany ) const {
  return replaceSubstring(m_pchData,pattern,replacement,casesens,howmany);
}

char CString::GetAt(int nIndex) const {
  ASSERT(nIndex >= 0);
  ASSERT(nIndex < GetData()->nDataLength);
  return m_pchData[nIndex];
}

char CString::operator[](int nIndex) const {
  ASSERT(nIndex >= 0);
  ASSERT(nIndex < GetData()->nDataLength);
  return m_pchData[nIndex];
}

void CString::SetAt(int nIndex, char ch) {
  ASSERT(nIndex >= 0);
  ASSERT(nIndex < GetData()->nDataLength);
  CopyBeforeWrite();
  m_pchData[nIndex] = ch;
}

CString::operator const char*() const {
  return m_pchData;
}

int CString::Compare(const char* lpsz) const {
  return strcmp(m_pchData,lpsz);
}

int CString::CompareNoCase(const char* lpsz) const {
  return strcasecmp(m_pchData,lpsz);
}

const CString& CString::operator=(const CString& stringSrc) {
  if (m_pchData != stringSrc.m_pchData) {
    if ((GetData()->nRefs < 0 && GetData() != _afxDataNil) ||
                        stringSrc.GetData()->nRefs < 0) {
      // actual copy necessary since one of the strings is locked
      AssignCopy(stringSrc.GetData()->nDataLength, stringSrc.m_pchData);
    }
    else {
      // can just copy references around
      Release();
      ASSERT(stringSrc.GetData() != _afxDataNil);
      m_pchData = stringSrc.m_pchData;
      InterlockedIncrement(&GetData()->nRefs);
    }
  }
  return *this;
}

const CString& CString::operator=(char ch) {
  AssignCopy(1,&ch);
  return *this;
}

const CString& CString::operator=(const char* lpsz) {
  AssignCopy(SafeStrlen(lpsz),lpsz);
  return *this;
}

const CString& CString::operator+=(const char* lpsz) {
  ConcatInPlace(SafeStrlen(lpsz), lpsz);
  return *this;
}

const CString& CString::operator+=(const CString& string) {
  ConcatInPlace(string.GetData()->nDataLength,string.m_pchData);
  return *this;
}

const CString& CString::operator+=(char ch) {
  ConcatInPlace(1,&ch);
  return *this;
}

CString::~CString() {
#if _DEBUG>1
  CStringData* sd = GetData();
  log("In CString::~CString(0x%X), GetData()=0x%X, refs=%d",this,sd, (sd==_afxDataNil) ? -1 : sd->nRefs );
#endif
  if (GetData() != _afxDataNil) {
    if( InterlockedDecrement(&GetData()->nRefs)<=0 )
      FreeData(GetData());
  }
#if _DEBUG>1
  log("Out CString::~CString(0x%X)",this);
#endif
}

int CString::GetAllocLength() const {
  return GetData()->nAllocLength;
}

CStringData* CString::GetData() const {
  ASSERT(m_pchData != 0);
  return ((CStringData*)m_pchData)-1;
}

void CString::Init() {
  m_pchData = afxEmptyString.m_pchData;
}

void CString::AllocCopy(
  CString& dest,
  int nCopyLen,
  int nCopyIndex,
  int nExtraLen) const {
  // will clone the data attached to this string
  // allocating 'nExtraLen' characters
  // Places results in uninitialized string 'dest'
  // Will copy the part or all of original data to start of new string
  int nNewLen = nCopyLen + nExtraLen;
  if (nNewLen == 0) {
    dest.Init();
  }
  else {
    dest.AllocBuffer(nNewLen);
    memcpy(dest.m_pchData, m_pchData+nCopyIndex, nCopyLen*sizeof(char));
  }
}

void CString::AllocBuffer(int nLen) {
  // always allocate one extra character for '\0' termination
  // assumes [optimistically] that data length will equal allocation length
  ASSERT(nLen >= 0);
  ASSERT(nLen <= 64534);    // max size (enough room for 1 extra)
  if (nLen == 0) {
    Init();
  }
  else {
    CStringData* pData;
    pData = (CStringData*)new BYTE[sizeof(CStringData) + (nLen+1)*sizeof(char)];
    pData->nAllocLength = nLen;
    pData->nRefs = 1;
    pData->data()[nLen] = '\0';
    pData->nDataLength = nLen;
    m_pchData = pData->data();
  }
}

void CString::AssignCopy(int nSrcLen, const char* lpszSrcData) {
  AllocBeforeWrite(nSrcLen);
  memcpy(m_pchData, lpszSrcData, nSrcLen*sizeof(char));
  GetData()->nDataLength = nSrcLen;
  m_pchData[nSrcLen] = '\0';
}

void CString::ConcatCopy(int nSrc1Len, const char* lpszSrc1Data, int nSrc2Len, const char* lpszSrc2Data) {
  int nNewLen = nSrc1Len + nSrc2Len;
  if (nNewLen != 0) {
    AllocBuffer(nNewLen);
    memcpy(m_pchData, lpszSrc1Data, nSrc1Len*sizeof(char));
    memcpy(m_pchData+nSrc1Len, lpszSrc2Data, nSrc2Len*sizeof(char));
  }
}

void CString::ConcatInPlace(int nSrcLen, const char* lpszSrcData) {
  //  -- the main routine for += operators
  // concatenating an empty string is a no-op!
  if (nSrcLen == 0)
    return;

  // if the buffer is too small, or we have a width mis-match, just
  // allocate a new buffer (slow but sure)
  if (GetData()->nRefs > 1 || GetData()->nDataLength + nSrcLen > GetData()->nAllocLength) {
    // we have to grow the buffer, use the ConcatCopy routine
    CStringData* pOldData = GetData();
    ConcatCopy(GetData()->nDataLength, m_pchData, nSrcLen, lpszSrcData);
    ASSERT(pOldData != 0);
    CString::Release(pOldData);
  }
  else {
    // fast concatenation when buffer big enough
    memcpy(m_pchData+GetData()->nDataLength, lpszSrcData,nSrcLen*sizeof(TCHAR));
    GetData()->nDataLength += nSrcLen;
    ASSERT(GetData()->nDataLength <= GetData()->nAllocLength);
    m_pchData[GetData()->nDataLength] = '\0';
  }
}

void CString::CopyBeforeWrite() {
  if (GetData()->nRefs > 1) {
    CStringData* pData = GetData();
    Release();
    AllocBuffer(pData->nDataLength);
    memcpy(m_pchData, pData->data(),(pData->nDataLength+1)*sizeof(TCHAR));
  }
  ASSERT(GetData()->nRefs <= 1);
}

void CString::AllocBeforeWrite(int nLen) {
  if (GetData()->nRefs > 1 || nLen > GetData()->nAllocLength) {
    Release();
    AllocBuffer(nLen);
  }
  ASSERT(GetData()->nRefs <= 1);
}

void CString::Release() {
  if (GetData() != _afxDataNil) {
    ASSERT(GetData()->nRefs != 0);
    if( InterlockedDecrement(&GetData()->nRefs)<=0 )
      FreeData(GetData());
    Init();
  }
}

void CString::Release(CStringData* pData) {
  if (pData != _afxDataNil) {
    ASSERT(pData->nRefs != 0);
    if( InterlockedDecrement(&pData->nRefs)<=0 )
      FreeData(pData);
  }
}

int  CString::SafeStrlen(const char* lpsz) {
  return (lpsz==0) ? 0 : strlen(lpsz);
}

void CString::FreeData(CStringData* pData) {
  delete[] (BYTE*)pData;
}

////////////////////////////////////////////////////////////////
// global scope string operators
////////////////////////////////////////////////////////////////
CString operator+(const CString& string1, const CString& string2) {
  CString s;
  s.ConcatCopy(string1.GetData()->nDataLength,string1.m_pchData,string2.GetData()->nDataLength,string2.m_pchData);
  return s;
}

CString operator+(const CString& string, char ch) {
  char ab[2] = {ch,0};
  return string+ab;
}

CString operator+(char ch, const CString& string) {
  char ab[2] = {ch,0};
  return ab+string;
}

CString operator+(const CString& string, const char* lpsz) {
  CString s;
  s.ConcatCopy(string.GetData()->nDataLength,string.m_pchData,CString::SafeStrlen(lpsz),lpsz);
  return s;
}

CString operator+(const char* lpsz, const CString& string) {
  CString s;
  s.ConcatCopy(CString::SafeStrlen(lpsz),lpsz,string.GetData()->nDataLength,string.m_pchData);
  return s;
}

// Compare helpers
int operator==(const CString& s1, const CString& s2) {
  return s1.Compare(s2)==0;
}

int operator==(const CString& s1, const char* s2) {
#ifdef HAVE_UGLY_NULL_FIX
  if( s2==NULL ) return s1.IsEmpty();
#endif
  return s1.Compare(s2)==0;
}

int operator==(const char* s1, const CString& s2) {
#ifdef HAVE_UGLY_NULL_FIX
  if( s1==NULL ) return s2.IsEmpty();
#endif
  return s2.Compare(s1) == 0;
}

int operator!=(const CString& s1, const CString& s2) {
  return s1.Compare(s2) != 0;
}

int operator!=(const CString& s1, const char* s2) {
#ifdef HAVE_UGLY_NULL_FIX
  if( s2==NULL ) return s1.IsNotEmpty();
#endif
  return s1.Compare(s2) != 0;
}

int operator!=(const char* s1, const CString& s2) {
#ifdef HAVE_UGLY_NULL_FIX
  if( s1==NULL ) return s2.IsEmpty();
#endif
  return s2.Compare(s1) != 0;
}

int operator<(const CString& s1, const CString& s2) {
  return s1.Compare(s2) < 0;
}

int operator<(const CString& s1, const char* s2) {
  return s1.Compare(s2) < 0;
}

int operator<(const char* s1, const CString& s2) {
  return s2.Compare(s1) > 0;
}

int operator>(const CString& s1, const CString& s2) {
  return s1.Compare(s2) > 0;
}

int operator>(const CString& s1, const char* s2) {
  return s1.Compare(s2) > 0;
}

int operator>(const char* s1, const CString& s2) {
  return s2.Compare(s1) < 0;
}

int operator<=(const CString& s1, const CString& s2) {
  return s1.Compare(s2) <= 0;
}

int operator<=(const CString& s1, const char* s2) {
  return s1.Compare(s2) <= 0;
}

int operator<=(const char* s1, const CString& s2) {
  return s2.Compare(s1) >= 0;
}

int operator>=(const CString& s1, const CString& s2) {
  return s1.Compare(s2) >= 0;
}

int operator>=(const CString& s1, const char* s2) {
  return s1.Compare(s2) >= 0;
}

int operator>=(const char* s1, const CString& s2) {
  return s2.Compare(s1) <= 0;
}

// Other string functions
bool compare_chars_case( char c1, char c2 ) {
        return c1==c2;
}

bool compare_chars_nocase( char c1, char c2 ) {
        return toupper(c1)==toupper(c2);
}

char* searchSubstring( const char* haystack, const char* needle, bool (*comparer)(char,char) ) {
        if( !haystack )
                return NULL;
        int matchedsofar = 0;
        int needtomatch  = strlen(needle);
        for( const char* s=haystack; *s; s++ ) {
                if( comparer(s[0],needle[matchedsofar]) ) {
                        if( ++matchedsofar==needtomatch ) {
                                return (char*)(s-needtomatch+1);
                        }
                }
                else {
                        matchedsofar = 0;
                }
        }
        return NULL;
}


CString replaceSubstring( const char* s, const char* pattern, const char* replacement, bool casesens, int howmany ) {
        CString result                  = "";
        bool (*comparer)(char,char)     = casesens ? compare_chars_case : compare_chars_nocase;
        for( int n=0; (howmany<0) || (n<howmany); n++ ) {
                char* found = searchSubstring(s,pattern,comparer);
                if( !found ) {
                        result += s;
                        break;
                }
                result += CString(s,found-s);
                result += replacement;
                s = found+strlen(pattern);
        }
        return result;
}

CString vformatString( const char* format, va_list ap ) {
        CString result;
        int size = 100;   // Guess we need no more than 100 bytes
        result.AllocBuffer(size);
        while (1) {
                /* Try to print in the allocated space. */
                int n = vsnprintf (result.m_pchData,size,format,ap);
                /* If that worked, return the string. */
                if (n > -1 && n < size)
                        break;
                /* Else try again with more space. */
                if (n > -1)    /* glibc 2.1 */
                        size = n+1; /* precisely what is needed */
                else           /* glibc 2.0 */
                        size *= 2;  /* twice the old size */
                result.AllocBuffer(size);
        }
        return result;
}

CString formatString( const char* format, ... ) {
        va_list ap;
        va_start(ap,format);
        CString result = vformatString(format,ap);
        va_end(ap);
        return result;
}

CString formatTime( time_t time, const char* format ) {
        char buff[256] = {0};
        struct tm tm;
        strftime(buff,sizeof(buff)-1,format,localtime_r(&time,&tm));
        return buff;
}

CString sqlEscapeString( const char* s ) {
        return replaceSubstring(s,"\'","\'\'");
}
CString quoteSqlString( const char* s ) {
        return s ? "\'"+sqlEscapeString(s)+"\'" : "NULL";
}
